@@ -1,5 +1,5 @@
-__version__ = "0.2.1"
+__version__ = "0.2.2"
 __author__ = 'Small Magellanic Cloud AI Ltd.'
 
 from smallcloud.code_root import detect_code_root, code_to_zip
-from smallcloud.launch import launch_task
+from smallcloud.launch import launch_task, shutdown
